require 'rails_helper'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

  describe Api::Dmo::RexController, :type => :api do
    routes {Hermes::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    context 'rex arborescence' do
      let!(:user1) { create(:user)}
      let!(:user2) { create(:user)}
  
      # A0
      # |
      # |
      # |
      # A1
      # |  \
      # |   \
      # |    \
      # A2   A21
      # | \    \
      # |  \    \
      # |   \    \
      # A3   A31  A32
      let!(:entityA0) { create(:entity, siret: 'A0000000000000', title: 'etpA0') }
      let!(:entityA1) { create(:entity, siret: 'A1000000000000', title: 'etpA1', parent_id: entityA0.id) }
      let!(:entityA2) { create(:entity, siret: 'A2000000000000', title: 'etpA2', parent_id: entityA1.id) }
      let!(:entityA21) { create(:entity, siret: 'A2100000000000', title: 'etpA21', parent_id: entityA1.id) }
      let!(:entityA3) { create(:entity, siret: 'A3000000000000', title: 'etpA3', parent_id: entityA2.id) }
      let!(:entityA31) { create(:entity, siret: 'A3100000000000', title: 'etp31', parent_id: entityA2.id) }
      let!(:entityA32) { create(:entity, siret: 'A3200000000000', title: 'etp32', parent_id: entityA21.id) }
      
      # 
      # B0
      # |
      # B1
      # |  \
      # B11 B12
      #      |
      #     B21
      let!(:entityB0) { create(:entity, siret: 'B0000000000000', title: 'etpB0') }
      let!(:entityB1) { create(:entity, siret: 'B1000000000000', title: 'etpB1', parent_id: entityB0.id) }
      let!(:entityB11) { create(:entity, siret: 'B1100000000000', title: 'etpB11', parent_id: entityB1.id) }
      let!(:entityB12) { create(:entity, siret: 'B1200000000000', title: 'etpB12', parent_id: entityB1.id) }
      let!(:entityB21) { create(:entity, siret: 'B2100000000000', title: 'etpB21', parent_id: entityB12.id) }


      describe 'GET root_elements' do
        let!(:entreprise_user) { create(:user, auth_roles:%w[entreprise], group_ids: [entityB1.supervisor_group_id, entityA1.supervisor_group_id]) }

        describe 'as user with acces' do
          before :each do
            ['A1', 'A2', 'A3'].each do |name|
              entity = eval("entity#{name}")
              user1.group_ids +=  [entity.group_id]
              user1.save
            end

            authenticated_header(request, user1)
            get 'root_elements'
          end

          it 'return valid data' do
            expect(response).to have_http_status(200)
            datas = json['data']
            expect(datas.map { |e| e['attributes']['title'] } ).to eq([entityA1.title])
          end
        end

        describe 'as user with acces on broken tree' do
          before :each do
            ['A2', 'A3', 'A21', 'A32'].each do |name|
              entity = eval("entity#{name}")
              user1.group_ids +=  [entity.group_id]
              user1.save
            end

            authenticated_header(request, user1)
            get 'root_elements'
          end

          it 'return valid data' do
            expect(response).to have_http_status(200)
            datas = json['data']
            expect(datas.map { |e| e['attributes']['title'] } ).to eq([entityA2.title, entityA21.title])
          end
        end

        describe 'with entreprise_user' do
          before :each do
            ['B1', 'B11', 'B12', 'A1', 'A2', 'A3'].each do |n|
              entity = eval("entity#{n}")
              entreprise_user.group_ids << entity.supervisor_group_id
              entreprise_user.save
            end
            authenticated_header(request, entreprise_user)
            get 'root_elements', {sort: {'title' => 1}}
          end

          it 'return valid data' do
            expect(response).to have_http_status(200)
            datas = json['data']
            ['B1', 'A1'].each do |n|
              expect(Hermes::Entity.readable_by(entreprise_user).find(eval("entity#{n}").id.to_s)).to_not be_nil
              expect(datas.any? { |e| e['attributes']['title'] ==  eval("entity#{n}").title} ).to eq(true)
            end
          end
        end

        describe 'with no access' do
          before :each do
            authenticated_header(request, user1)
            get 'root_elements'
          end
          
          it 'return valid data' do
            expect(response).to have_http_status(200)
            datas = json['data']
            expect(datas).to eq([])
          end
        end
        
        describe 'for_lessor without bailleur' do
          let!(:entreprise) { create(:user, auth_roles:%w[entreprise]) }
          before :each do
            authenticated_header(request, entreprise)
            get 'root_elements', {for_lessor: true}
          end

          it 'return error' do
            expect(response).to have_http_status(422)
          end
        end

        describe 'for_lessor with bailleur' do
          let!(:lessor) { create(:lessor, siret: 'B0000000000000', title: 'lessor') }
          let!(:user_bailleur) { create(:user, auth_roles: ['bailleur'], group_ids: [lessor.supervisor_group_id]) }

          before :each do
            authenticated_header(request, user_bailleur)
            get 'root_elements', {for_lessor: true}
          end

          it 'return valid data' do
            datas = json['data']
            expect(response).to have_http_status(200)
            expect(Hermes::Lessor.readable_by(user_bailleur).find(lessor.id.to_s)).to_not be_nil
            expect(datas[0]['attributes']['title']).to eq(lessor.title)
          end
        end
      end

      describe 'GET show' do
        describe 'as user with acces' do
          before :each do
            ['A0', 'A1', 'A2', 'A21', 'A3', 'A31', 'A32', 'B11', 'B12'].each do |name|
              entity = eval("entity#{name}")
              entity.readers = [user1.id.to_s]
              entity.save
            end

            authenticated_header(request, user1)
            get 'show', {id: entityA0.id.to_s, max_depth: 5, parent_depth: 5}
          end

          it 'return valid data' do
            expect(response).to have_http_status(200)
            datas = json['data']
            expect(datas['from_element']['id'] ).to eq(entityA0.id.to_s)
            expect(datas['tree'].length ).to eq(1)

            def recursive_titles(parent, acc)
              acc << parent['title']
              if parent['children']
                parent['children'].each do |children| 
                  recursive_titles(children, acc)
                end
              end
              acc
            end

            children_titles = recursive_titles(datas['tree'][0], [datas['tree'][0]['title']])

            ['A1', 'A2', 'A21', 'A3', 'A31', 'A32'].each do |name| 
              entity = eval("entity#{name}")
              expect(children_titles.include?(entity.title) ).to eq(true)
            end
          end
        end
      end

      describe 'GET subsidiaries' do
        describe 'as user with access' do
          ['A0', 'A1', 'A2', 'A21', 'A3', 'A32', 'B1', 'B11'].each_with_index do |name, i|
            let!("user_right_#{i}") { create(
              :user_right,
              user: user1,
              entity: eval("entity#{name}"),
              right: 'administrator',
              type: 'entity'
            )}
          end

          before :each do
            authenticated_header(request, user1)
            get 'subsidiaries', {
              id: entityA1.id.to_s
            }
          end

          it 'return valid data' do
            names = ["etpA1", "etpA2", "etpA21", "etpA3", "etp32"]
            titles = json['data'].map { |e| e['attributes']['title'] } 
            expect(response).to have_http_status(200)
            expect(titles).to eq(names)
          end
        end
      end
    end
  end
  
  # describe Api::Rex::RexController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
  #   require_relative '../../swagger_helper'
  #   TAGS_API_REX= 'API_REX_REX'
  #   include JwtAuthHelper
  #   extend HermesSwaggerJsonParams
  #   include SwaggerJsonAttributes

  #   let!(:user) { create(:user) }
  #   let!(:entreprise) { create(:user, auth_roles: %w[entreprise], current_auth_role: "entreprise") }
  #   let!(:bailleur) { create(:user, auth_roles: %w[bailleur], current_auth_role: "bailleur") }

  #   context 'as entreprise' do
  #     path '/api/rex/root_elements' do
  #       get 'find root elements' do
  #         tags TAGS_API_REX
  #         produces 'application/json'
  #         consumes 'application/json'
  #         security [Bearer: {}]
  #         parameter(
  #           name: :for_lessor,
  #           in: :query,
  #           type: :string
  #         )

  #         response '200', 'root elements list' do
  #           let(:Authorization) {"Bearer #{token_for(entreprise)}"}
  #           let(:params) do
  #           end
  #           include_context 'with integration test'
  #           include_context 'valid jsonapi'
  #           include_context 'not containings errors'

  #           after do |_example|
  #             json = JSON.parse(response.body)

  #             expect(json['data']).to be_a Hash
  #             expect(json['data'].length).to be == 0
  #           end
  #         end
  #       end
  #     end
  #   end
  # end
end
