require 'rails_helper'

RSpec.describe(Hermes::Applicant, type: :model) do
  it 'has a valid factory' do
    expect(FactoryBot.create(:applicant)).to be_valid
  end
  
  it 'has a valid with_eligible_offers trait' do
    applicant = FactoryBot.create(:applicant, :with_eligible_offers, offers_count: 3, salary: 2620, children_count: 3, maried: true)
    offers_family2 = Hermes::HousingOffer.where(applicant.reload.eligibility_query.json_to_h)
    expect(offers_family2.count).to eq(3)
  end

  let!(:user1) { create(:user) }

  describe 'relations' do
    let!(:entity1) { create(:entity, siret: 'test1000000000') }
    let!(:applicant) { create(:applicant, main_employer_siret: 'test1000000000', user_id: user1.id, entity: entity1) }
    let!(:applicant2) { create(:applicant) }
    let!(:housing_request) { create(:housing_request, main_applicant: applicant, is_active: true) }

    
    it 'put referent' do
      applicant_link = Hermes::ApplicantLink.create(type: 'link', applicant_ids: [applicant, applicant2].map { |elt| elt.id.to_s }, housing_request: housing_request)
      expect(Hermes::ApplicantLink.where(applicant_ids: applicant.id.to_s).count).to eq(1)
      expect(applicant.reload.links.count).to eq(1)
      expect(applicant.link_types_for(applicant2.id)).to eq(['link'])
      expect(applicant.entity.id).to eq entity1.id
      expect(applicant.reload.readers).to include "g_#{entity1.supervisor_group.id}"
      expect(applicant.reload.writers).to include "g_#{entity1.manager_group.id}"
    end

    describe 'changing applicant main_employer_siret' do
      let!(:entity2) { create(:entity, siret: 'test2000000000') }
      let!(:user_entity1) { create(:user, group_ids: [entity1.manager_group.id]) }
      let!(:user_entity2) { create(:user, group_ids: [entity2.manager_group.id]) }

      before :each do
        applicant.update(main_employer_siret: 'test2000000000')
      end

      it 'applicant readers/writers updated' do
        r_applicant = applicant.reload
        expect(r_applicant.entity.id).to eq entity2.id
        expect(r_applicant.readers).not_to include "g_#{entity1.supervisor_group.id}"
        expect(r_applicant.writers).not_to include "g_#{entity1.manager_group.id}"
        expect(r_applicant.readers).to include "g_#{entity2.supervisor_group.id}"
        expect(r_applicant.writers).to include "g_#{entity2.manager_group.id}"
      end
      
      it 'housing_request readers/writers updated' do
        r_housing_request = housing_request.reload
        expect(r_housing_request.entity_id).to eq entity2.id.to_s
        expect(r_housing_request.readable_by?(user_entity1)).to eq false
        expect(r_housing_request.readable_by?(user_entity2)).to eq true
      end

      it 'housing_request readers/writers removed' do
        applicant.update(main_employer_siret: nil)
        r_housing_request = housing_request.reload
        expect(r_housing_request.entity_id).to eq nil
        expect(r_housing_request.readable_by?(user_entity1)).to eq false
        expect(r_housing_request.readable_by?(user_entity2)).to eq false
      end

    end
  end

  # BEFORE ACTION UNIT TESTS
  context 'before action unit test context' do
    # WHERE : we create an applicant with an user and a first active request
    let!(:user) { create(:user, first_name: 'Michel', last_name: 'Dupont', email: 'michel.dupont@gmail.com') }
    let!(:user2) { create(:user, first_name: 'Jackie', last_name: 'Mike', email: 'jackie.mike@gmail.com') }
    let!(:entity) { create(:entity, {title: 'title1', siret: '12345600000000'}) }
    let!(:applicant) { create(:applicant, user: user, entity: entity, status: 'OK', monthly_incomes: {drug_selling: 20}, monthly_charges: {dog_food: 5}) }
    let!(:applicant2) { create(:applicant, user: user2, entity: entity, status: 'OK', monthly_incomes: {porn: 30}, monthly_charges: {cat_food: 10}) }
    let!(:housing_request) { create(:housing_request, 
      main_applicant: applicant, 
      coapplicants: [{rel_type: 'marié', id: applicant2.id.to_s}], 
      dependents: [{rel_type: 'enfant à charge', monthly_incomes: {pokemon_selling: 5}, monthly_charges: {unicorn_food: 10}}], 
      is_active: true
    ) }

    #update_employer_data
    describe 'update_employer_data unit test' do
      let!(:entity2) {create(:entity, {title: 'title2', siret: '78900000000000'})}
      before :each do
        # WHEN : entity_id_changed?
        applicant.entity = entity2
        applicant.save
      end
      #TEHN : we werify the users data has changed
      it 'employer data has not changed' do
        expect(applicant.reload.main_employer_name).not_to eq('title2')
        expect(applicant.main_employer_siret).not_to eq('78900000000000')
      end
    end

    #update_birth_dependencies
    describe 'update_birth_dependencies' do
      before :each do
        # WHEN : birth date_changed?
        applicant.birth_date = Date.new(1918, 12, 12)
        applicant.save
      end
      #TEHN : we verify the users data has changed
      it 'set the birth_plus_30 correctly' do
        expect(applicant.birth_plus_30.to_datetime).to eq((Date.new(1918, 12, 12) + 30.years).to_datetime)
      end
    end

    #update_status_dependencies
    describe 'update_status_dependencies' do
      before :each do
        applicant.status_updated_at = (Time.current - 1.day).to_datetime
        applicant.save
        # WHEN : status_changed?
        applicant.status = 'KO'
        applicant.save
      end
      #TEHN : we verify the users data has changed
      it 'set the status_updated_at to now' do
        expect(applicant.status_updated_at).to be > (Time.now - 10.minute).to_datetime
      end
    end

    #check_all
    describe 'check_all' do
      #check_monthly_revenues_dependency
      describe 'check_monthly_revenues_dependency' do
        describe 'change in monthly_incomes' do
          before :each do
            applicant.monthly_incomes = { drug_selling: 10 }
            applicant.save
          end
          it 'has made changes in first_active_request' do
            reloaded_hr = housing_request.reload
            expect(reloaded_hr.monthly_revenues).to eq(20)
          end
        end
        describe 'change in monthly_charges' do
          before :each do
            applicant.monthly_charges = {dog_food: 0} # ON MET KOI A DEDANS???
            applicant.save
          end

          it 'has made changes in first_active_request' do
            reloaded_hr = housing_request.reload
            expect(reloaded_hr.monthly_revenues).to eq(35)
          end
        end
        describe 'change in monthly_charges coapplicant' do
          before :each do
            applicant2.update(monthly_incomes: {
              'Salaire': 1000,
            })
          end

          it 'has made changes in coapplicant request' do
            reloaded_hr = housing_request.reload
            expect(reloaded_hr.applicant_ids).to eq([applicant2.id.to_s])
            expect(reloaded_hr.monthly_revenues).to eq(1000)
          end
        end
      end
      #check_category_dependency
      describe 'check_category_dependency' do
        before :each do
          applicant.family_situation = 'relation libre polyamoureuse'
        end
        #we test if should_update_eligibility before save
        it 'self.should_update_eligibility = true' do
          applicant.before_actions
          expect(applicant.should_update_eligibility).to eq(true)
          applicant.save
          expect(housing_request.reload.category_may_change).to eq(false)
          expect(applicant.reload.should_update_eligibility).to eq(nil)
        end
      end

      #check_eligibility_dependency
      describe 'check_eligibility_dependency' do
        %w[
          entity_id
          mutation
          mobility
          priority_public
          single_parent
        ].concat((1..12).map {|i| "treshold_category_#{i}"}).each do |changed_field|
          describe 'when changing ' + changed_field.to_s + 'on applicant'
          before :each do
            applicant[changed_field] = !applicant[changed_field]
          end
          it 'has trigger housing_request check' do
            applicant.before_actions
            expect(applicant.should_update_eligibility).to eq(true)
            applicant.save
            expect(housing_request.reload.category_may_change).to eq(false)
            expect(applicant.reload.should_update_eligibility).to eq(nil)
          end
        end
      end
    end

    #check_year_revenues
    describe 'check_year revenue dependency' do
      before :each do
        applicant.amount_year_minus_two = 20000
      end
      #we test if should_update_eligibility before save
      it 'self.should_update_eligibility = true' do
        applicant.save
        expect(housing_request.reload.tax_revenues_will_change).to eq(false)
        expect(applicant.reload.should_update_eligibility).to eq(nil)
        
        applicant.amount_year_minus_two = 10000
        applicant.save
        expect(housing_request.reload.reference_tax_revenues).to eq(10000)
        expect(applicant.reload.eligibility_query.json_to_h[:ceil_category_3]).to eq({"$gte"=>10000.0})
      end
    end
  end


  context 'check on eligibility for different cases' do

    let!(:housing_offer_L1) do
      create(
        :housing_offer, {
          :typology=>"T3",
          :district=>"Antony",
          :department=>"92",
          :rent_amount=>640.0,
          :rent_with_charges=>690.0,
          :surface=>57,
          :rooms=>3,
          :availability_date=>'14 Sep 2019 00:00:00 +0200',
          :description=>"Près tous commerces.",
          :reserved=>false,
          :address=>"23 avenue de Vaugirard",
          :threshold_type=>"PLUS",
          :threshold_subtype=>"IDF hors Paris et Communes limitrophes",
          :rental_charges=>50.0,
          :lessor_status=>"PUBLIE",
          :offer_status=>"LOGEMENT PUBLIE",
          :ceil_category_1=>23721.0,
          :ceil_category_2=>35452.0,
          :ceil_category_3=>42616.0,
          :ceil_category_4=>51046.0,
          :ceil_category_5=>60429.0,
          :ceil_category_6=>68001.0,
          :ceil_category_7=>75578.0,
          :ceil_category_8=>83155.0,
          :ceil_category_9=>90732.0,
          :ceil_category_10=>98309.0,
          :ceil_category_11=>105886.0,
          :ceil_category_12=>113463.0,
          :floor_category_1=>0.0,
          :floor_category_2=>0.0,
          :floor_category_3=>0.0,
          :floor_category_4=>0.0,
          :floor_category_5=>0.0,
          :floor_category_6=>0.0,
          :floor_category_7=>0.0,
          :floor_category_8=>0.0,
          :floor_category_9=>0.0,
          :floor_category_10=>0.0,
          :floor_category_11=>0.0,
          :floor_category_12=>0.0,
          :min_revenues=>2072.072072072072,
          :total_charges=>50.0,
          :min_effort_rate=>0.333,
          :administration_area=>"Grand Centre",
          :latitude =>2.28921,
          :longitude =>48.7534,
          :lessor_name=>"ESPACE HABITAT CONSTRUC.",
          :lessor_siren=>"987654355",
          :postal_code=>"92160",
          :reserving_business_name=>"ALS",
          :designation_date_end=>'13 Dec 2019 23:00:00 +0100',
          :commercialized=>false,
          :is_mad=>true,
          :is_mad_updated_at=>'06 Nov 2019 16:52:59 +0100',
          :max_family_size=>4,
          :couple_min_family_size=>1,
          :single_min_family_size=>1
        }
      )
    end

    let!(:housing_offer_L2) do
      create(
        :housing_offer, {
          :typology=>"T3",
          :district=>"Sceaux",
          :department=>"92",
          :rent_amount=>750.0,
          :rent_with_charges=>810.0,
          :surface=>53,
          :rooms=>3,
          :availability_date=>'14 Sep 2019 00:00:00 +0200',
          :description=>"Près tous commerces.",
          :reserved=>false,
          :address=>"8 rue des Bons Enfants",
          :threshold_type=>"PLUS",
          :threshold_subtype=>"IDF hors Paris et Communes limitrophes",
          :rental_charges=>60.0,
          :lessor_status=>"PUBLIE",
          :offer_status=>"LOGEMENT PUBLIE",
          :ceil_category_1=>23721.0,
          :ceil_category_2=>35452.0,
          :ceil_category_3=>42616.0,
          :ceil_category_4=>51046.0,
          :ceil_category_5=>60429.0,
          :ceil_category_6=>68001.0,
          :ceil_category_7=>75578.0,
          :ceil_category_8=>83155.0,
          :ceil_category_9=>90732.0,
          :ceil_category_10=>98309.0,
          :ceil_category_11=>105886.0,
          :ceil_category_12=>113463.0,
          :floor_category_1=>0.0,
          :floor_category_2=>0.0,
          :floor_category_3=>0.0,
          :floor_category_4=>0.0,
          :floor_category_5=>0.0,
          :floor_category_6=>0.0,
          :floor_category_7=>0.0,
          :floor_category_8=>0.0,
          :floor_category_9=>0.0,
          :floor_category_10=>0.0,
          :floor_category_11=>0.0,
          :floor_category_12=>0.0,
          :min_revenues=>2432.4324324324325,
          :total_charges=>60.0,
          :min_effort_rate=>0.333,
          :administration_area=>"Grand Centre",
          :latitude =>2.2879923,
          :longitude =>48.7788137,
          :lessor_name=>"ESPACE HABITAT CONSTRUC.",
          :lessor_siren=>"987654355",
          :postal_code=>"92330",
          :reserving_business_name=>"ALS",
          :designation_date_end=>'13 Dec 2019 23:00:00 +0100',
          :commercialized=>false,
          :is_mad=>true,
          :is_mad_updated_at=>'06 Nov 2019 16:58:21 +0100',
          :max_family_size=>4,
          :couple_min_family_size=>1,
          :single_min_family_size=>1
        }
      )
    end

    let!(:housing_offer_L3) do
      create(
        :housing_offer, {
          :typology=>"T3",
          :district=>"Antony",
          :department=>"92",
          :rent_amount=>560.0,
          :rent_with_charges=>650.0,
          :surface=>70,
          :rooms=>3,
          :availability_date=>'14 Sep 2019 00:00:00 +0200',
          :description=>"Près tous commerces.",
          :reserved=>false,
          :address=>"45 rue des Bornes",
          :threshold_type=>"PLA I",
          :threshold_subtype=>"IDF hors Paris et Communes limitrophes",
          :rental_charges=>90.0,
          :lessor_status=>"PUBLIE",
          :offer_status=>"LOGEMENT PUBLIE",
          :ceil_category_1=>13050.0,
          :ceil_category_2=>21272.0,
          :ceil_category_3=>25569.0,
          :ceil_category_4=>28075.0,
          :ceil_category_5=>33238.0,
          :ceil_category_6=>37401.0,
          :ceil_category_7=>41567.0,
          :ceil_category_8=>45733.0,
          :ceil_category_9=>49899.0,
          :ceil_category_10=>54065.0,
          :ceil_category_11=>58231.0,
          :ceil_category_12=>62397.0,
          :floor_category_1=>0.0,
          :floor_category_2=>0.0,
          :floor_category_3=>0.0,
          :floor_category_4=>0.0,
          :floor_category_5=>0.0,
          :floor_category_6=>0.0,
          :floor_category_7=>0.0,
          :floor_category_8=>0.0,
          :floor_category_9=>0.0,
          :floor_category_10=>0.0,
          :floor_category_11=>0.0,
          :floor_category_12=>0.0,
          :min_revenues=>1951.9519519519517,
          :total_charges=>90.0,
          :min_effort_rate=>0.333,
          :administration_area=>"Grand Centre",
          :latitude =>2.2906231,
          :longitude =>48.7597488,
          :lessor_name=>"ESPACE HABITAT CONSTRUC.",
          :lessor_siren=>"987654355",
          :postal_code=>"92160",
          :reserving_business_name=>"ALS",
          :designation_date_end=>'13 Dec 2019 23:00:00 +0100',
          :commercialized=>false,
          :is_mad=>true,
          :is_mad_updated_at=>'06 Nov 2019 16:56:00 +0100',
          :max_family_size=>4,
          :couple_min_family_size=>1,
          :single_min_family_size=>1
        }
      )
    end

    let!(:housing_offer_L4) do
      create(
        :housing_offer, {
          :typology=>"T2",
          :district=>"Bourg La Reine",
          :department=>"92",
          :rent_amount=>900.0,
          :rent_with_charges=>1010.0,
          :surface=>80,
          :rooms=>2,
          :availability_date=>'14 Sep 2019 00:00:00 +0200',
          :description=>"Près tous commerces.",
          :reserved=>false,
          :address=>"29 rue Eiffel",
          :threshold_type=>"PLUS",
          :threshold_subtype=>"IDF hors Paris et Communes limitrophes",
          :rental_charges=>110.0,
          :lessor_status=>"PUBLIE",
          :offer_status=>"LOGEMENT PUBLIE",
          :ceil_category_1=>23721.0,
          :ceil_category_2=>35452.0,
          :ceil_category_3=>42616.0,
          :ceil_category_4=>51046.0,
          :ceil_category_5=>60429.0,
          :ceil_category_6=>68001.0,
          :ceil_category_7=>75578.0,
          :ceil_category_8=>83155.0,
          :ceil_category_9=>90732.0,
          :ceil_category_10=>98309.0,
          :ceil_category_11=>105886.0,
          :ceil_category_12=>113463.0,
          :floor_category_1=>0.0,
          :floor_category_2=>0.0,
          :floor_category_3=>0.0,
          :floor_category_4=>0.0,
          :floor_category_5=>0.0,
          :floor_category_6=>0.0,
          :floor_category_7=>0.0,
          :floor_category_8=>0.0,
          :floor_category_9=>0.0,
          :floor_category_10=>0.0,
          :floor_category_11=>0.0,
          :floor_category_12=>0.0,
          :min_revenues=>3033.033033033033,
          :total_charges=>110.0,
          :min_effort_rate=>0.333,
          :administration_area=>"Grand Centre",
          :latitude =>2.3183651,
          :longitude =>48.7769129,
          :lessor_name=>"ESPACE HABITAT CONSTRUC.",
          :lessor_siren=>"987654355",
          :postal_code=>"92340",
          :reserving_business_name=>"ALS",
          :designation_date_end=>'13 Dec 2019 23:00:00 +0100',
          :commercialized=>false,
          :is_mad=>true,
          :is_mad_updated_at=>'06 Nov 2019 15:53:48 +0100',
          :max_family_size=>3,
          :couple_min_family_size=>1,
          :single_min_family_size=>1
        }
      )
    end

    let!(:housing_offer_L5) do
      create(
        :housing_offer, {
          :typology=>"T3",
          :district=>"Boulogne Billancourt",
          :department=>"92",
          :rent_amount=>1000.0,
          :rent_with_charges=>1110.0,
          :surface=>80,
          :rooms=>3,
          :availability_date=>'14 Sep 2019 00:00:00 +0200',
          :description=>"Près tous commerces.",
          :reserved=>false,
          :address=>"56 rue des Amandiers",
          :threshold_type=>"PLUS",
          :threshold_subtype=>"Paris et communes limitrophes",
          :rental_charges=>110.0,
          :lessor_status=>"PUBLIE",
          :offer_status=>"LOGEMENT PUBLIE",
          :ceil_category_1=>23721.0,
          :ceil_category_2=>35452.0,
          :ceil_category_3=>46473.0,
          :ceil_category_4=>55486.0,
          :ceil_category_5=>66017.0,
          :ceil_category_6=>74286.0,
          :ceil_category_7=>82564.0,
          :ceil_category_8=>90842.0,
          :ceil_category_9=>99120.0,
          :ceil_category_10=>107398.0,
          :ceil_category_11=>115676.0,
          :ceil_category_12=>123954.0,
          :floor_category_1=>0.0,
          :floor_category_2=>0.0,
          :floor_category_3=>0.0,
          :floor_category_4=>0.0,
          :floor_category_5=>0.0,
          :floor_category_6=>0.0,
          :floor_category_7=>0.0,
          :floor_category_8=>0.0,
          :floor_category_9=>0.0,
          :floor_category_10=>0.0,
          :floor_category_11=>0.0,
          :floor_category_12=>0.0,
          :min_revenues=>3333.333333333333,
          :total_charges=>110.0,
          :min_effort_rate=>0.333,
          :administration_area=>"Grand Centre",
          :latitude =>2.23716,
          :longitude =>48.8458,
          :lessor_name=>"ESPACE HABITAT CONSTRUC.",
          :lessor_siren=>"987654355",
          :postal_code=>"92100",
          :reserving_business_name=>"ALS",
          :designation_date_end=>'13 Dec 2019 23:00:00 +0100',
          :commercialized=>false,
          :is_mad=>true,
          :is_mad_updated_at=>'06 Nov 2019 15:57:06 +0100',
          :max_family_size=>4,
          :couple_min_family_size=>1,
          :single_min_family_size=>1
        }
      )
    end

    let!(:housing_offer_L6) do
      create(
        :housing_offer, {
          :typology=>"T3",
          :district=>"Boulogne Billancourt",
          :department=>"92",
          :rent_amount=>700.0,
          :rent_with_charges=>810.0,
          :surface=>85,
          :rooms=>3,
          :availability_date=>'14 Sep 2019 00:00:00 +0200',
          :description=>"Près tous commerces.",
          :reserved=>false,
          :address=>"12 avenue Cartier",
          :threshold_type=>"PLA I",
          :threshold_subtype=>"IDF hors Paris et Communes limitrophes",
          :rental_charges=>110.0,
          :lessor_status=>"PUBLIE",
          :offer_status=>"LOGEMENT PUBLIE",
          :ceil_category_1=>13050.0,
          :ceil_category_2=>21272.0,
          :ceil_category_3=>25569.0,
          :ceil_category_4=>28075.0,
          :ceil_category_5=>33238.0,
          :ceil_category_6=>37401.0,
          :ceil_category_7=>41567.0,
          :ceil_category_8=>45733.0,
          :ceil_category_9=>49899.0,
          :ceil_category_10=>54065.0,
          :ceil_category_11=>58231.0,
          :ceil_category_12=>62397.0,
          :floor_category_1=>0.0,
          :floor_category_2=>0.0,
          :floor_category_3=>0.0,
          :floor_category_4=>0.0,
          :floor_category_5=>0.0,
          :floor_category_6=>0.0,
          :floor_category_7=>0.0,
          :floor_category_8=>0.0,
          :floor_category_9=>0.0,
          :floor_category_10=>0.0,
          :floor_category_11=>0.0,
          :floor_category_12=>0.0,
          :min_revenues=>2432.4324324324325,
          :total_charges=>110.0,
          :min_effort_rate=>0.333,
          :administration_area=>"Grand Centre",
          :latitude =>2.24042,
          :longitude =>48.8474,
          :lessor_name=>"ESPACE HABITAT CONSTRUC.",
          :lessor_siren=>"987654355",
          :postal_code=>"92100",
          :reserving_business_name=>"ALS",
          :designation_date_end=>'13 Dec 2019 23:00:00 +0100',
          :commercialized=>false,
          :is_mad=>true,
          :is_mad_updated_at=>'06 Nov 2019 15:59:16 +0100',
          :max_family_size=>4,
          :couple_min_family_size=>1,
          :single_min_family_size=>1
        }
      )
    end

    let!(:housing_offer_L7) do
      create(
        :housing_offer, {
          :typology=>"T3",
          :district=>"Sceaux",
          :department=>"92",
          :rent_amount=>500.0,
          :rent_with_charges=>610.0,
          :surface=>90,
          :rooms=>3,
          :availability_date=>'14 Sep 2019 00:00:00 +0200',
          :description=>"Près tous commerces.",
          :reserved=>false,
          :address=>"12 avenue du Maine",
          :threshold_type=>"PLA I",
          :threshold_subtype=>"IDF hors Paris et Communes limitrophes",
          :rental_charges=>110.0,
          :lessor_status=>"PUBLIE",
          :offer_status=>"LOGEMENT PUBLIE",
          :ceil_category_1=>13050.0,
          :ceil_category_2=>21272.0,
          :ceil_category_3=>25569.0,
          :ceil_category_4=>28075.0,
          :ceil_category_5=>33238.0,
          :ceil_category_6=>37401.0,
          :ceil_category_7=>41567.0,
          :ceil_category_8=>45733.0,
          :ceil_category_9=>49899.0,
          :ceil_category_10=>54065.0,
          :ceil_category_11=>58231.0,
          :ceil_category_12=>62397.0,
          :floor_category_1=>0.0,
          :floor_category_2=>0.0,
          :floor_category_3=>0.0,
          :floor_category_4=>0.0,
          :floor_category_5=>0.0,
          :floor_category_6=>0.0,
          :floor_category_7=>0.0,
          :floor_category_8=>0.0,
          :floor_category_9=>0.0,
          :floor_category_10=>0.0,
          :floor_category_11=>0.0,
          :floor_category_12=>0.0,
          :min_revenues=>1831.8318318318318,
          :total_charges=>110.0,
          :min_effort_rate=>0.333,
          :administration_area=>"Grand Centre",
          :latitude =>2.29923,
          :longitude =>48.7784,
          :lessor_name=>"ESPACE HABITAT CONSTRUC.",
          :lessor_siren=>"987654355",
          :postal_code=>"92330",
          :reserving_business_name=>"ALS",
          :designation_date_end=>'13 Dec 2019 23:00:00 +0100',
          :commercialized=>false,
          :is_mad=>true,
          :is_mad_updated_at=>'06 Nov 2019 16:01:35 +0100',
          :max_family_size=>4,
          :couple_min_family_size=>1,
          :single_min_family_size=>1
        }
      )
    end

    let!(:housing_offer_L8) do
      create(
        :housing_offer, {
          :typology=>"T4",
          :district=>"Sceaux",
          :department=>"92",
          :rent_amount=>1450.0,
          :rent_with_charges=>1560.0,
          :surface=>95,
          :rooms=>4,
          :availability_date=>'14 Sep 2019 00:00:00 +0200',
          :description=>"Près tous commerces.",
          :reserved=>false,
          :address=>"26 avenue du Maine",
          :threshold_type=>"PLUS",
          :threshold_subtype=>"IDF hors Paris et Communes limitrophes",
          :rental_charges=>110.0,
          :lessor_status=>"PUBLIE",
          :offer_status=>"LOGEMENT PUBLIE",
          :ceil_category_1=>23721.0,
          :ceil_category_2=>35452.0,
          :ceil_category_3=>42616.0,
          :ceil_category_4=>51046.0,
          :ceil_category_5=>60429.0,
          :ceil_category_6=>68001.0,
          :ceil_category_7=>75578.0,
          :ceil_category_8=>83155.0,
          :ceil_category_9=>90732.0,
          :ceil_category_10=>98309.0,
          :ceil_category_11=>105886.0,
          :ceil_category_12=>113463.0,
          :floor_category_1=>0.0,
          :floor_category_2=>0.0,
          :floor_category_3=>0.0,
          :floor_category_4=>0.0,
          :floor_category_5=>0.0,
          :floor_category_6=>0.0,
          :floor_category_7=>0.0,
          :floor_category_8=>0.0,
          :floor_category_9=>0.0,
          :floor_category_10=>0.0,
          :floor_category_11=>0.0,
          :floor_category_12=>0.0,
          :min_revenues=>4684.684684684685,
          :total_charges=>110.0,
          :min_effort_rate=>0.333,
          :administration_area=>"Grand Centre",
          :latitude =>2.28849,
          :longitude =>48.7759,
          :lessor_name=>"ESPACE HABITAT CONSTRUC.",
          :lessor_siren=>"987654355",
          :postal_code=>"92330",
          :reserving_business_name=>"ALS",
          :designation_date_end=>'13 Dec 2019 23:00:00 +0100',
          :commercialized=>false,
          :is_mad=>true,
          :is_mad_updated_at=>'06 Nov 2019 16:03:53 +0100',
          :max_family_size=>5,
          :couple_min_family_size=>2,
          :single_min_family_size=>2
        }
      )
    end

    let!(:housing_offer_L9) do
      create(
        :housing_offer, {
          :typology=>"T3",
          :district=>"Paris",
          :department=>"75",
          :rent_amount=>760.0,
          :rent_with_charges=>810.0,
          :surface=>45,
          :rooms=>3,
          :availability_date=>'14 Sep 2019 00:00:00 +0200',
          :description=>"Près tous commerces.",
          :reserved=>false,
          :address=>"13 avenue du Maine",
          :threshold_type=>"PLA I",
          :threshold_subtype=>"Paris et communes limitrophes",
          :rental_charges=>50.0,
          :lessor_status=>"PUBLIE",
          :offer_status=>"LOGEMENT PUBLIE",
          :ceil_category_1=>13050.0,
          :ceil_category_2=>21272.0,
          :ceil_category_3=>27883.0,
          :ceil_category_4=>30521.0,
          :ceil_category_5=>36307.0,
          :ceil_category_6=>40859.0,
          :ceil_category_7=>45411.0,
          :ceil_category_8=>49963.0,
          :ceil_category_9=>54515.0,
          :ceil_category_10=>59067.0,
          :ceil_category_11=>63619.0,
          :ceil_category_12=>68171.0,
          :floor_category_1=>0.0,
          :floor_category_2=>0.0,
          :floor_category_3=>0.0,
          :floor_category_4=>0.0,
          :floor_category_5=>0.0,
          :floor_category_6=>0.0,
          :floor_category_7=>0.0,
          :floor_category_8=>0.0,
          :floor_category_9=>0.0,
          :floor_category_10=>0.0,
          :floor_category_11=>0.0,
          :floor_category_12=>0.0,
          :min_revenues=>2432.4324324324325,
          :total_charges=>50.0,
          :min_effort_rate=>0.333,
          :administration_area=>"Grand Centre",
          :latitude =>2.29658,
          :longitude =>48.8341,
          :lessor_name=>"ESPACE HABITAT CONSTRUC.",
          :lessor_siren=>"987654355",
          :postal_code=>"75015",
          :reserving_business_name=>"ALS",
          :designation_date_end=>'13 Dec 2019 23:00:00 +0100',
          :commercialized=>false,
          :is_mad=>true,
          :is_mad_updated_at=>'06 Nov 2019 16:08:32 +0100',
          :max_family_size=>4,
          :couple_min_family_size=>1,
          :single_min_family_size=>1
        }
      )
    end

    let!(:housing_offer_L10) do
      create(
        :housing_offer, {
          :typology=>"T3",
          :district=>"Paris",
          :department=>"75",
          :rent_amount=>760.0,
          :rent_with_charges=>810.0,
          :surface=>45,
          :rooms=>3,
          :availability_date=>'14 Sep 2019 00:00:00 +0200',
          :description=>"Près tous commerces.",
          :reserved=>false,
          :address=>"27 avenue du Maine",
          :threshold_type=>"PLUS",
          :threshold_subtype=>"Paris et communes limitrophes",
          :rental_charges=>50.0,
          :lessor_status=>"PUBLIE",
          :offer_status=>"LOGEMENT PUBLIE",
          :ceil_category_1=>23721.0,
          :ceil_category_2=>35452.0,
          :ceil_category_3=>46473.0,
          :ceil_category_4=>55486.0,
          :ceil_category_5=>66017.0,
          :ceil_category_6=>74286.0,
          :ceil_category_7=>82564.0,
          :ceil_category_8=>90842.0,
          :ceil_category_9=>99120.0,
          :ceil_category_10=>107398.0,
          :ceil_category_11=>115676.0,
          :ceil_category_12=>123954.0,
          :floor_category_1=>0.0,
          :floor_category_2=>0.0,
          :floor_category_3=>0.0,
          :floor_category_4=>0.0,
          :floor_category_5=>0.0,
          :floor_category_6=>0.0,
          :floor_category_7=>0.0,
          :floor_category_8=>0.0,
          :floor_category_9=>0.0,
          :floor_category_10=>0.0,
          :floor_category_11=>0.0,
          :floor_category_12=>0.0,
          :min_revenues=>2432.4324324324325,
          :total_charges=>50.0,
          :min_effort_rate=>0.333,
          :administration_area=>"Grand Centre",
          :latitude =>2.36228,
          :longitude =>48.8933,
          :lessor_name=>"ESPACE HABITAT CONSTRUC.",
          :lessor_siren=>"987654355",
          :postal_code=>"75015",
          :reserving_business_name=>"ALS",
          :designation_date_end=>'13 Dec 2019 23:00:00 +0100',
          :commercialized=>false,
          :is_mad=>true,
          :is_mad_updated_at=>'06 Nov 2019 16:10:15 +0100',
          :max_family_size=>4,
          :couple_min_family_size=>1,
          :single_min_family_size=>1
        }
      )
    end

    let!(:housing_offer_L11) do
      create(
        :housing_offer, {
          :typology=>"T2",
          :district=>"Paris",
          :department=>"75",
          :rent_amount=>760.0,
          :rent_with_charges=>810.0,
          :surface=>45,
          :rooms=>2,
          :availability_date=>'14 Sep 2019 00:00:00 +0200',
          :description=>"Près tous commerces.",
          :reserved=>false,
          :address=>"27 avenue du Maine",
          :threshold_type=>"PLS 1/97 (sans dérogation)",
          :threshold_subtype=>"Paris et communes limitrophes",
          :rental_charges=>50.0,
          :lessor_status=>"PUBLIE",
          :offer_status=>"LOGEMENT PUBLIE",
          :ceil_category_1=>23721.0,
          :ceil_category_2=>35452.0,
          :ceil_category_3=>46473.0,
          :ceil_category_4=>55486.0,
          :ceil_category_5=>66017.0,
          :ceil_category_6=>74286.0,
          :ceil_category_7=>82564.0,
          :ceil_category_8=>90842.0,
          :ceil_category_9=>99120.0,
          :ceil_category_10=>107398.0,
          :ceil_category_11=>115676.0,
          :ceil_category_12=>123954.0,
          :floor_category_1=>0.0,
          :floor_category_2=>0.0,
          :floor_category_3=>0.0,
          :floor_category_4=>0.0,
          :floor_category_5=>0.0,
          :floor_category_6=>0.0,
          :floor_category_7=>0.0,
          :floor_category_8=>0.0,
          :floor_category_9=>0.0,
          :floor_category_10=>0.0,
          :floor_category_11=>0.0,
          :floor_category_12=>0.0,
          :min_revenues=>2432.4324324324325,
          :total_charges=>50.0,
          :min_effort_rate=>0.333,
          :administration_area=>"Grand Centre",
          :latitude =>2.4031013,
          :longitude =>48.8675617,
          :lessor_name=>"ESPACE HABITAT CONSTRUC.",
          :lessor_siren=>"987654355",
          :postal_code=>"75015",
          :reserving_business_name=>"ALS",
          :designation_date_end=>'13 Dec 2019 23:00:00 +0100',
          :commercialized=>false,
          :is_mad=>true,
          :is_mad_updated_at=>'06 Nov 2019 16:11:53 +0100',
          :max_family_size=>3,
          :couple_min_family_size=>1,
          :single_min_family_size=>1
        }
      )
    end

    let!(:housing_offer_L26) do
      create(
        :housing_offer, {
          :typology=>"T2",
          :district=>"Bourg La Reine",
          :department=>"92",
          :rent_amount=>300.0,
          :rent_with_charges=>330.0,
          :surface=>80,
          :rooms=>2,
          :availability_date=>'14 Sep 2019 00:00:00 +0200',
          :description=>"Près tous commerces.",
          :reserved=>false,
          :address=>"56 rue des Amandiers",
          :threshold_type=>"PLA I",
          :threshold_subtype=>"IDF hors Paris et Communes limitrophes",
          :rental_charges=>30.0,
          :lessor_status=>"PUBLIE",
          :offer_status=>"LOGEMENT PUBLIE",
          :ceil_category_1=>13050.0,
          :ceil_category_2=>21272.0,
          :ceil_category_3=>25569.0,
          :ceil_category_4=>28075.0,
          :ceil_category_5=>33238.0,
          :ceil_category_6=>37401.0,
          :ceil_category_7=>41567.0,
          :ceil_category_8=>45733.0,
          :ceil_category_9=>49899.0,
          :ceil_category_10=>54065.0,
          :ceil_category_11=>58231.0,
          :ceil_category_12=>62397.0,
          :floor_category_1=>0.0,
          :floor_category_2=>0.0,
          :floor_category_3=>0.0,
          :floor_category_4=>0.0,
          :floor_category_5=>0.0,
          :floor_category_6=>0.0,
          :floor_category_7=>0.0,
          :floor_category_8=>0.0,
          :floor_category_9=>0.0,
          :floor_category_10=>0.0,
          :floor_category_11=>0.0,
          :floor_category_12=>0.0,
          :min_revenues=>990.9909909909909,
          :total_charges=>30.0,
          :min_effort_rate=>0.333,
          :administration_area=>"Grand Centre",
          :latitude =>2.316,
          :longitude =>48.7811,
          :lessor_name=>"ESPACE HABITAT CONSTRUC.",
          :lessor_siren=>"987654355",
          :postal_code=>"92340",
          :reserving_business_name=>"ALS",
          :designation_date_end=>'13 Dec 2019 23:00:00 +0100',
          :commercialized=>false,
          :is_mad=>true,
          :is_mad_updated_at=>'15 Nov 2019 19:06:22 +0100',
          :max_family_size=>3,
          :couple_min_family_size=>1,
          :single_min_family_size=>1
        }
      )
    end

    let!(:housing_offer_NR1) do
      create(
        :housing_offer, {
          :typology=>"T2",
          :type=>"Maison",
          :district=>"Toulouse",
          :department=>"31",
          :rent_amount=>230.0,
          :rent_with_charges=>280.0,
          :surface=>40,
          :rooms=>2,
          :availability_date=>'12 Sep 2019 00:00:00 +0200',
          :description=>"Près tous commerces.",
          :address=>"15 boulevard Jean Rostand",
          :threshold_type=>"PLUS",
          :threshold_subtype=>"IDF hors Paris et Communes limitrophes",
          :rental_charges=>50.0,
          :lessor_status=>"PUBLIE",
          :offer_status=>"LOGEMENT PUBLIE",
          :ceil_category_1=>23721.0,
          :ceil_category_2=>35452.0,
          :ceil_category_3=>42616.0,
          :ceil_category_4=>51046.0,
          :ceil_category_5=>60429.0,
          :ceil_category_6=>68001.0,
          :ceil_category_7=>75578.0,
          :ceil_category_8=>83155.0,
          :ceil_category_9=>90732.0,
          :ceil_category_10=>98309.0,
          :ceil_category_11=>105886.0,
          :ceil_category_12=>113463.0,
          :floor_category_1=>0.0,
          :floor_category_2=>0.0,
          :floor_category_3=>0.0,
          :floor_category_4=>0.0,
          :floor_category_5=>0.0,
          :floor_category_6=>0.0,
          :floor_category_7=>0.0,
          :floor_category_8=>0.0,
          :floor_category_9=>0.0,
          :floor_category_10=>0.0,
          :floor_category_11=>0.0,
          :floor_category_12=>0.0,
          :min_revenues=>840.8408408408408,
          :total_charges=>50.0,
          :min_effort_rate=>0.333,
          :administration_area=>"Grand Ouest",
          :lessor_name=>"CASSIOPAE HABITAT",
          :lessor_siren=>"326820065",
          :postal_code=>"83100",
          :reserving_business_name=>"ALS",
          :designation_date_end=>'11 Dec 2019 23:00:00 +0100',
          :entities_siret=>["9876543251", "123456789"],
          :commercialized=>false,
          :is_mad=>true,
          :is_mad_updated_at=>'04 Nov 2019 12:39:59 +0100',
          :max_family_size=>3,
          :couple_min_family_size=>1,
          :single_min_family_size=>1
        }
      )
    end
    
    let!(:housing_offer_NR2) do
      create(
        :housing_offer, {
          :typology=>"T3",
          :district=>"Toulon",
          :department=>"83",
          :rent_amount=>640.0,
          :rent_with_charges=>690.0,
          :surface=>57,
          :rooms=>3,
          :availability_date=>'12 Sep 2019 00:00:00 +0200',
          :description=>"Près tous commerces.",
          :reserved=>true,
          :address=>"15 boulevard Jean Rostand",
          :threshold_type=>"PLUS",
          :threshold_subtype=>"PLUS IDF Hors Paris",
          :rental_charges=>50.0,
          :lessor_status=>"PUBLIE",
          :offer_status=>"LOGEMENT PUBLIE",
          :ceil_category_1=>23721.0,
          :ceil_category_2=>35452.0,
          :ceil_category_3=>42616.0,
          :ceil_category_4=>51046.0,
          :ceil_category_5=>60429.0,
          :ceil_category_6=>68001.0,
          :ceil_category_7=>75578.0,
          :ceil_category_8=>83155.0,
          :ceil_category_9=>90732.0,
          :ceil_category_10=>98309.0,
          :ceil_category_11=>105886.0,
          :ceil_category_12=>113463.0,
          :floor_category_1=>0.0,
          :floor_category_2=>0.0,
          :floor_category_3=>0.0,
          :floor_category_4=>0.0,
          :floor_category_5=>0.0,
          :floor_category_6=>0.0,
          :floor_category_7=>0.0,
          :floor_category_8=>0.0,
          :floor_category_9=>0.0,
          :floor_category_10=>0.0,
          :floor_category_11=>0.0,
          :floor_category_12=>0.0,
          :min_revenues=>2072.072072072072,
          :total_charges=>50.0,
          :min_effort_rate=>0.333,
          :administration_area=>"Grand Ouest",
          :latitude =>5.8664113,
          :longitude =>43.1100206,
          :lessor_name=>"CASSIOPAE HABITAT",
          :lessor_siren=>"326820065",
          :postal_code=>"83100",
          :reserving_business_name=>"ALS",
          :designation_date_end=>'11 Dec 2019 23:00:00 +0100',
          :entities_siret=>["54210557200631"],
          :commercialized=>false,
          :is_mad=>true,
          :is_mad_updated_at=>'05 Nov 2019 11:18:39 +0100',
          :max_family_size=>4,
          :couple_min_family_size=>1,
          :single_min_family_size=>1
        }
      )
    end

    let!(:housing_offer_NR3) do
      create(
        :housing_offer, {
          :typology=>"T3",
          :district=>"Fréjus",
          :department=>"83",
          :rent_amount=>420.0,
          :rent_with_charges=>450.0,
          :surface=>80,
          :rooms=>3,
          :availability_date=>'13 Dec 2019 23:00:00 +0100',
          :description=>"Près tous commerces.",
          :reserved=>true,
          :address=>"56 avenue de Provence",
          :threshold_type=>"PLA I",
          :threshold_subtype=>"IDF hors Paris et Communes limitrophes",
          :rental_charges=>30.0,
          :lessor_status=>"PUBLIE",
          :offer_status=>"LOGEMENT PUBLIE",
          :ceil_category_1=>13050.0,
          :ceil_category_2=>21272.0,
          :ceil_category_3=>25569.0,
          :ceil_category_4=>28075.0,
          :ceil_category_5=>33238.0,
          :ceil_category_6=>37401.0,
          :ceil_category_7=>41567.0,
          :ceil_category_8=>45733.0,
          :ceil_category_9=>49899.0,
          :ceil_category_10=>54065.0,
          :ceil_category_11=>58231.0,
          :ceil_category_12=>62397.0,
          :floor_category_1=>0.0,
          :floor_category_2=>0.0,
          :floor_category_3=>0.0,
          :floor_category_4=>0.0,
          :floor_category_5=>0.0,
          :floor_category_6=>0.0,
          :floor_category_7=>0.0,
          :floor_category_8=>0.0,
          :floor_category_9=>0.0,
          :floor_category_10=>0.0,
          :floor_category_11=>0.0,
          :floor_category_12=>0.0,
          :min_revenues=>1351.3513513513512,
          :total_charges=>30.0,
          :min_effort_rate=>0.333,
          :administration_area=>"Grand Centre",
          :lessor_name=>"ESPACE HABITAT CONSTRUC.",
          :lessor_siren=>"987654355",
          :postal_code=>"83600",
          :reserving_business_name=>"ALS",
          :designation_date_end=>'25 Dec 2019 23:00:00 +0100',
          :entities_siret=>["40333593800193"],
          :commercialized=>false,
          :is_mad=>true,
          :is_mad_updated_at=>'18 Nov 2019 14:50:39 +0100',
          :max_family_size=>4,
          :couple_min_family_size=>1,
          :single_min_family_size=>1
        }
      )
    end

    let!(:user2b) { create(:user) }
    let!(:applicant2b) do 
      create(
        :applicant,
        user: user2b,
        salary: 1300,
        amount_year_minus_one: 0,
        amount_year_minus_two: 0,
        work_contract_type: "CDIP"
      )
    end

    let!(:user2a) { create(:user) }
    let!(:applicant2a) do 
      create(
        :applicant,
        :with_eligible_offers,
        maried: true,
        offers_count: 0,
        user: user2a,
        salary: 1300,
        year_minus_one: 31200.0,
        year_minus_two: 31200.0,
        work_contract_type: "CDI"
      )
    end

    let!(:housing_request2) do
      create(
        :housing_request,
        user_id: user2a.id, 
        is_active: true, 
        priority_points_indexes: [], 
        has_priority: false, 
        reference_tax_revenues: 31200.0, 
        dependents: [
          {
            "last_name"=>"GALLET", 
            "first_name"=>"ANTONY", 
            "birth_date"=>"2017-03-23T00:00:00.000Z", 
            "rel_type"=>"E", 
            "sex"=>"M"
          }
        ], 
        category_type: 3, 
        single_parent: false, 
        widen_request: true, 
        housing_type_list: ["RE"], 
        seeked_district_postal_codes: [
          "92330", 
          "92160", 
          "92340", 
          "92100"
        ], 
        seeked_departments: [
          "92", 
          "92", 
          "92", 
          "92", 
          "75"
        ], 
        seeked_localisation_list: [
          {
            "nom"=>"Sceaux", 
            "code"=>"92071", 
            "codeDepartement"=>"92", 
            "codeRegion"=>"11", 
            "codesPostaux"=>["92330"], 
            "population"=>19479, 
            "_score"=>1
          },{
            "nom"=>"Antony", 
            "code"=>"92002", 
            "codeDepartement"=>"92", 
            "codeRegion"=>"11", 
            "codesPostaux"=>["92160"], 
            "population"=>62210, 
            "_score"=>0.21006442878421805
          },{
            "nom"=>"Bourg-la-Reine", 
            "code"=>"92014", 
            "codeDepartement"=>"92", 
            "codeRegion"=>"11", 
            "codesPostaux"=>["92340"], 
            "population"=>20531, 
            "_score"=>0.03486614664709854
          },{
            "nom"=>"Boulogne-Billancourt", 
            "code"=>"92012", 
            "codeDepartement"=>"92", 
            "codeRegion"=>"11", 
            "codesPostaux"=>["92100", 
            "75016"], 
            "population"=>119645, 
            "_score"=>0.6337893925744263
          },{
            "nom"=>"Paris", 
            "code"=>"75", 
            "codeRegion"=>"11", 
            "_score"=>1
          }], 
        requester_firstname: "Charles", 
        requester_lastname: "GALLET", 
        requester_fullname: "Charles GALLET", 
        requester_email: "c.om.ptet.e.stals@gmail.com", 
        requester_birth_date: "1995-03-12 00:00:00 UTC", 
        monthly_revenues: 2600.0, 
        typology_range: ["T2", "T3", "T4"], 
        family_size: 3, 
        unborn_children: [{}], 
        actual_housing_details: {}, 
        postal_details: {
          "address_number"=>"122", 
          "address_street"=>"bis rue de Sèvres", 
          "postal_code"=>"92100", 
          "address_insee_code"=>"BOULOGNE BILLANCOURT"
        }, 
        owned_housing_details: {
          "type"=>nil, 
          "postal_code"=>nil, 
          "district"=>nil
        }, 
        employee_id: "MLLS", 
        employee_start_date: "2019-01-01 00:00:00 UTC", 
        entity_access_level: "total", 
        tab_state: "0", 
        motivation1: "02", 
        motivation2: "02", 
        motivation3: "02", 
        seeked_housing_categories: [""], 
        min_surface: 25, 
        dependents_monthly_incomes: {}, 
        dependents_monthly_charges: {}, 
        out_of_range_types: ["T2", "T3", "T4"], 
        applications_nb: 0, 
        name_has_changed: false, 
        birth_date_has_changed: false, 
        monthly_revenues_will_change: false, 
        category_may_change: false, 
        should_update_eligibility: false, 
        tax_revenues_will_change: false, 
        housing_offer_ids: [], 
        entity_status: "A VALIDER", 
        request_status: "ELIGIBLE", 
        nud_creations: {}, 
        main_applicant_id: applicant2a, 
        applicant_ids: [applicant2b.id], 
      )
    end

    let!(:userGOUBILL) { create(:user) }
    let!(:raphaelGOUBILL) do 
      create(
        :applicant,
        :with_eligible_offers,
        offers_count: 0,
        user: userGOUBILL,
        salary: 2500,
        amount_year_minus_one: 22000.0,
        amount_year_minus_two: 22000.0,
        family_situation: "D",
        work_contract_type: "AGP"
      )
    end

    let!(:housing_requestGOUBILL) do
      create(
        :housing_request,
        user_id: userGOUBILL.id, 
        als_id: "f0decb85a83b3df337", 
        is_active: true,                  
        priority_points_indexes: [], 
        has_priority: false,    
        reference_tax_revenues: 25000.0, 
        dependents: [{"last_name"=>"GOUBILL", 
        "first_name"=>"CHRIS", 
        "birth_date"=>"2017-03-23T00:00:00.000Z", 
        "rel_type"=>"droit de visite", 
        "sex"=>"M"}], 
        category_type: 1, 
        single_parent: false,           
        widen_request: true, 
        housing_type_list: ["RSLL"],    
        seeked_district_postal_codes: ["92330", 
        "92160", 
        "75001", 
        "92340", 
        "92100"],  
        seeked_departments: ["92", 
        "92", 
        "75", 
        "92", 
        "92"], 
        seeked_localisation_list: [{"nom"=>"Sceaux", 
        "code"=>"92071", 
        "codeDepartement"=>"92", 
        "codeRegion"=>"11", 
        "codesPostaux"=>["92330"], 
        "population"=>19479, 
        "_score"=>1}, 
        {"nom"=>"Antony", 
        "code"=>"92002", 
        "codeDepartement"=>"92", 
        "codeRegion"=>"11", 
        "codesPostaux"=>["92160"], 
        "population"=>62210, 
        "_score"=>0.21006442878421805}, 
        {"nom"=>"Paris", 
        "code"=>"75056", 
        "codeDepartement"=>"75", 
        "codeRegion"=>"11", 
        "codesPostaux"=>["75001", 
        "75002", 
        "75003", 
        "75004", 
        "75005", 
        "75006", 
        "75007", 
        "75008", 
        "75009", 
        "75010", 
        "75011", 
        "75012", 
        "75013", 
        "75014", 
        "75015", 
        "75016", 
        "75017", 
        "75018", 
        "75019", 
        "75020"], 
        "population"=>2190327, 
        "_score"=>0.29658920463148064}, 
        {"nom"=>"Bourg-la-Reine", 
        "code"=>"92014", 
        "codeDepartement"=>"92", 
        "codeRegion"=>"11", 
        "codesPostaux"=>["92340"], 
        "population"=>20531, 
        "_score"=>0.1047225184514744}, 
        {"nom"=>"Boulogne-Billancourt", 
        "code"=>"92012", 
        "codeDepartement"=>"92", 
        "codeRegion"=>"11", 
        "codesPostaux"=>["92100", 
        "75016"], 
        "population"=>119645, 
        "_score"=>0.6337893925744263}],  
        requester_firstname: "Raphael", 
        requester_lastname: "GOUBILL", 
        requester_fullname: "Raphael GOUBILL", 
        requester_email: "c.ompte.te.s.tals@gmail.com", 
        requester_birth_date: '1978-03-12 00:00:00 UTC', 
        monthly_revenues: 2500.0,  
        typology_range: ["T2", 
        "T3"], 
        family_size: 2, 
        unborn_children: [{}],   
        social_housing_entry_date: '2016-12-02 00:00:00 UTC',    
        actual_housing_details: {
          "postal_code"=>"75008", 
          "number"=>"14", 
          "way"=>"rue Montalivet", 
          "place"=>"paris"
        }, 
        postal_details: {}, 
        owned_housing_details: {
          "type"=>nil, 
          "postal_code"=>nil, 
          "district"=>nil
        }, 
        employee_id: "M121212", 
        employee_start_date: '2019-01-01 00:00:00 UTC', 
        entity_access_level: "total", 
        tab_state: "0", 
        motivation1: "13", 
        seeked_housing_categories: [""], 
        min_surface: 16, 
        dependents_monthly_incomes: {}, 
        dependents_monthly_charges: {}, 
        out_of_range_types: ["T2", 
        "T3"], 
        applications_nb: 0, 
        name_has_changed: false, 
        birth_date_has_changed: false, 
        monthly_revenues_will_change: false, 
        category_may_change: false, 
        should_update_eligibility: false, 
        tax_revenues_will_change: false, 
        superfields: {}, 
        applicant_ids: [], 
        housing_offer_ids: [], 
        entity_status: "A VALIDER", 
        request_status: "COMPLETE", 
        nud_creations: {}, 
        main_applicant_id: raphaelGOUBILL.id, 
      )
    end

    let!(:userGATTIN) { create(:user) }
    let!(:manonGATTIN) do 
      create(
        :applicant,
        :with_eligible_offers,
        maried: false,
        offers_count: 0,
        user: userGATTIN,
        salary: 1200,
        amount_year_minus_one: 0.0,
        amount_year_minus_two: 0.0,
        work_contract_type: "CDD"
      )
    end

    let!(:housing_requestGATTIN) do
      create(
        :housing_request,
        user_id: userGATTIN.id, 
        als_id: "7251444fecee390b78", 
        is_active: true, 
        has_priority: false, 
        reference_tax_revenues: 0.0, 
        category_type: 1, 
        single_parent: false, 
        widen_request: true, 
        housing_type_list: ["HLM"], 
        seeked_district_postal_codes: ['75000'], 
        seeked_localisation_list: [{"nom"=>"Paris", 
        "code"=>"75013",
        "codesPostaux"=>['75000'],
        'codeDepartement'=>'75',
        "_score"=>1}], 
        requester_firstname: "Manon", 
        requester_lastname: "GATTIN", 
        requester_fullname: "Manon GATTIN", 
        requester_email: "c.omp.te.testal.s@gmail.com", 
        requester_birth_date: '1990-12-03 00:00:00 UTC', 
        monthly_revenues: 1200.0, 
        entity_id: "", 
        typology_range: ["T1", 
        "T2"], 
        family_size: 1, 
        unborn_children: [{}], 
        actual_housing_details: {}, 
        postal_details: {"address_number"=>"21 ", 
        "address_street"=>"quai d’Austerlitz ", 
        "postal_code"=>"75013", 
        "address_insee_code"=>"Paris"}, 
        owned_housing_details: {
          "type"=>nil, 
          "postal_code"=>nil, 
          "district"=>nil
        }, 
        employee_id: "A021456", 
        employee_start_date: '2017-12-03 00:00:00 UTC', 
        entity_access_level: "total", 
        tab_state: "0", 
        motivation1: "01", 
        motivation2: "02", 
        motivation3: "04", 
        seeked_housing_categories: ["A"], 
        min_surface: 9, 
        dependents_monthly_incomes: {}, 
        dependents_monthly_charges: {}, 
        out_of_range_types: ["T1", 
        "T2"], 
        applications_nb: 0, 
        name_has_changed: false, 
        birth_date_has_changed: false, 
        monthly_revenues_will_change: false, 
        category_may_change: false, 
        should_update_eligibility: false, 
        tax_revenues_will_change: false, 
        superfields: {}, 
        entity_status: "A VALIDER", 
        request_status: "INCOMPLETE", 
        nud_creations: {}, 
        main_applicant_id: manonGATTIN.id, 
      )
    end

    let!(:userGILLET) { create(:user) }
    let!(:laureGILLET) do 
      create(
        :applicant,
        :with_eligible_offers,
        offers_count: 0,
        user: userGILLET,
        salary: 3300,
        amount_year_minus_one: 16500.0,
        amount_year_minus_two: 16500.0,
        work_contract_type: "CDIP"
      )
    end

    let!(:housing_requestGILLET) do
      create(
        :housing_request,
        user_id: userGILLET.id, 
        is_active: true, 
        has_priority: false, 
        reference_tax_revenues: 16500.0, 
        category_type: 1, 
        single_parent: false, 
        widen_request: true, 
        housing_type_list: ["RE"], 
        seeked_district_postal_codes: ["92330", 
        "92160", 
        "92340", 
        "92100"], 
        seeked_departments: ["92", 
        "92", 
        "92", 
        "92",
        "75"],
        seeked_localisation_list: [{"nom"=>"Sceaux", 
        "code"=>"92071", 
        "codeDepartement"=>"92", 
        "codeRegion"=>"11", 
        "codesPostaux"=>["92330"], 
        "population"=>19479, 
        "_score"=>0.5875338694313645}, 
        {"nom"=>"Antony", 
        "code"=>"92002", 
        "codeDepartement"=>"92", 
        "codeRegion"=>"11", 
        "codesPostaux"=>["92160"], 
        "population"=>62210, 
        "_score"=>0.42327771946153253}, 
        {"nom"=>"Bourg-la-Reine", 
        "code"=>"92014", 
        "codeDepartement"=>"92", 
        "codeRegion"=>"11", 
        "codesPostaux"=>["92340"], 
        "population"=>20531, 
        "_score"=>0.1047225184514744}, 
        {"nom"=>"Boulogne-Billancourt", 
        "code"=>"92012", 
        "codeDepartement"=>"92", 
        "codeRegion"=>"11", 
        "codesPostaux"=>["92100", 
        "75016"], 
        "population"=>119645, 
        "_score"=>0.6337893925744263}, 
        {"nom"=>"Paris", 
        "code"=>"75", 
        "codeRegion"=>"11", 
        "_score"=>1}], 
        requester_firstname: "Laure", 
        requester_lastname: "GILLET", 
        requester_fullname: "Laure GILLET", 
        requester_email: "c.ompt.e.tes.tals@gmail.com", 
        requester_birth_date: '1985-08-11 23:00:00 UTC', 
        monthly_revenues: 3300.0, 
        entity_id: "5dd3a27aa70bf2255538a310", 
        typology_range: ["T1", 
        "T2"], 
        family_size: 1, 
        unborn_children: [{}], 
        actual_housing_details: {}, 
        postal_details: {"address_number"=>"784", 
        "address_street"=>"ROUTE DE MONTAGNE", 
        "postal_code"=>"45000", 
        "address_insee_code"=>"ORLEANS"}, 
        owned_housing_details: {
          "type"=>nil, 
          "postal_code"=>nil, 
          "district"=>nil
        }, 
        employee_id: "MSMSLS", 
        employee_start_date: '2019-01-01 00:00:00 UTC', 
        entity_access_level: "total", 
        tab_state: "0", 
        motivation1: "02", 
        motivation2: "02", 
        motivation3: "02", 
        seeked_housing_categories: [""], 
        min_surface: 9, 
        dependents_monthly_incomes: {}, 
        dependents_monthly_charges: {}, 
        out_of_range_types: ["T1", 
        "T2"], 
        applications_nb: 0, 
        name_has_changed: false, 
        birth_date_has_changed: false, 
        monthly_revenues_will_change: false, 
        category_may_change: false, 
        should_update_eligibility: false, 
        tax_revenues_will_change: false, 
        superfields: {}, 
        entity_status: "A VALIDER", 
        request_status: "EN ATTENTE DE PIECES", 
        nud_creations: {}, 
        main_applicant_id: laureGILLET.id, 
      )
    end

    let!(:userGAUVIN) { create(:user) }
    let!(:cassandraGAUVAIN) do
      create(
        :applicant,
        :with_eligible_offers,
        maried: false,
        offers_count: 0,
        user: userGAUVIN,
        salary: 2500,
        amount_year_minus_one: 25000.0,
        amount_year_minus_two: 25000.0,
        work_contract_type: "CDIP"
      )
    end

    let!(:userDURANDETa) { create(:user) }
    let!(:userDURANDETb) { create(:user) }
    let!(:simonDURANDET) do
      create(
        :applicant,
        :with_eligible_offers,
        maried: false,
        offers_count: 0,
        user: userDURANDETa,
        salary: 1400,
        amount_year_minus_one: 31800.0,
        amount_year_minus_two: 31800.0,
        work_contract_type: "CDIP"
      )
    end
    let!(:yvonneDURANDET) do
      create(
        :applicant,
        :with_eligible_offers,
        maried: false,
        offers_count: 0,
        user: userDURANDETb,
        salary: 1250,
        amount_year_minus_one: 0.0,
        amount_year_minus_two: 0.0,
        work_contract_type: "CDIP"
      )
    end

    let!(:housing_requestDURANDET) do
      create(
        :housing_request,
        user_id: userDURANDETa.id, 
        is_active: true, 
        priority_points_indexes: [], 
        has_priority: false, 
        reference_tax_revenues: 31800.0, 
        dependents: [{"last_name"=>"DURANDET", 
        "first_name"=>"ANTONY", 
        "birth_date"=>"2017-03-30T23:00:00.000Z", 
        "rel_type"=>"P", 
        "sex"=>"M"}, 
        {"last_name"=>"DURANDET", 
        "first_name"=>"ROSE", 
        "birth_date"=>"2019-08-08T23:00:00.000Z", 
        "rel_type"=>"E", 
        "sex"=>"F"}], 
        seeked_districts: ["Sceaux", 
        "Antony", 
        "Boulogne-Billancourt", 
        "Bourg-la-Reine", 
        "Paris"], 
        category_type: 2, 
        single_parent: false, 
        widen_request: true, 
        housing_type_list: ["HLM"], 
        seeked_district_postal_codes: ["92330", 
        "92160", 
        "92100", 
        "92340"], 
        seeked_departments: ["92",
        "92",
        "92",
        "92",
        "75"],
        seeked_localisation_list: [{"nom"=>"Sceaux", 
        "code"=>"92071", 
        "codeDepartement"=>"92", 
        "codeRegion"=>"11", 
        "codesPostaux"=>["92330"], 
        "population"=>19479, 
        "_score"=>1}, 
        {"nom"=>"Antony", 
        "code"=>"92002", 
        "codeDepartement"=>"92", 
        "codeRegion"=>"11", 
        "codesPostaux"=>["92160"], 
        "population"=>62210, 
        "_score"=>0.21006442878421805}, 
        {"nom"=>"Boulogne-Billancourt", 
        "code"=>"92012", 
        "codeDepartement"=>"92", 
        "codeRegion"=>"11", 
        "codesPostaux"=>["92100", 
        "75016"], 
        "population"=>119645, 
        "_score"=>0.6337893925744263}, 
        {"nom"=>"Bourg-la-Reine", 
        "code"=>"92014", 
        "codeDepartement"=>"92", 
        "codeRegion"=>"11", 
        "codesPostaux"=>["92340"], 
        "population"=>20531, 
        "_score"=>0.1047225184514744}, 
        {"nom"=>"Paris", 
        "code"=>"75", 
        "codeRegion"=>"11", 
        "_score"=>1}], 
        requester_firstname: "Simon", 
        requester_lastname: "DURANDET", 
        requester_fullname: "Simon DURANDET", 
        requester_email: "com.ptete.stal.s@gmail.com", 
        requester_birth_date: '1978-12-14 00:00:00 UTC', 
        monthly_revenues: 2650.0, 
        typology_range: ["T3", 
        "T4", 
        "T5"], 
        family_size: 4, 
        unborn_children: [{}], 
        actual_housing_details: {}, 
        postal_details: {"address_number"=>"78", 
        "address_street"=>"ter rue de paris", 
        "postal_code"=>"92130", 
        "address_insee_code"=>"ISSY LES MOULINEAUX"}, 
        owned_housing_details: {
          "type"=>nil, 
          "postal_code"=>nil, 
          "district"=>nil
        }, 
        employee_id: "MSKSL", 
        employee_start_date: '2019-01-01 00:00:00 UTC', 
        entity_access_level: "total", 
        tab_state: "0", 
        motivation1: "02", 
        motivation2: "02", 
        motivation3: "02", 
        seeked_housing_categories: [""], 
        min_surface: 34, 
        dependents_monthly_incomes: {}, 
        dependents_monthly_charges: {}, 
        out_of_range_types: ["T3", 
        "T4", 
        "T5"], 
        applications_nb: 0, 
        name_has_changed: false, 
        birth_date_has_changed: false, 
        monthly_revenues_will_change: false, 
        category_may_change: false, 
        should_update_eligibility: false, 
        tax_revenues_will_change: false, 
        superfields: {"last_coapplicants"=>[{"rel_type"=>"M", 
        "id"=>"5df3b30ba70bf20c640361dd"}]}, 
        applicant_ids: [yvonneDURANDET.id], 
        housing_offer_ids: [], 
        entity_status: "A VALIDER", 
        request_status: "ELIGIBLE", 
        nud_creations: {}, 
        main_applicant_id: simonDURANDET.id, 
      )
    end

    let!(:userRAHMAa) { create(:user) }
    let!(:userRAHMAb) { create(:user) }
    let!(:julieRAHMA) do
      create(
        :applicant,
        :with_eligible_offers,
        maried: false,
        offers_count: 0,
        user: userRAHMAa,
        salary: 1450,
        amount_year_minus_one: 33900.0,
        amount_year_minus_two: 33900.0,
        work_contract_type: "CDIP"
      )
    end
    let!(:ambroiseRAHMA) do
      create(
        :applicant,
        :with_eligible_offers,
        maried: false,
        offers_count: 0,
        user: userRAHMAb,
        salary: 1375,
        amount_year_minus_one: 0.0,
        amount_year_minus_two: 0.0,
        work_contract_type: "CDIP"
      )
    end

    let!(:housing_requestRAHMA) do
      create(
        :housing_request,
        user_id: userRAHMAa.id,
        als_id: "4dd095a7c9451a1298",
        is_active: true,
        priority_points_indexes: [],
        has_priority: false,
        reference_tax_revenues: 33900.0,
        dependents: [{"last_name"=>"RAHMA",
        "first_name"=>"TITOUAN",
        "birth_date"=>"2017-06-30T23:00:00.000Z",
        "rel_type"=>"E",
        "sex"=>"M"}],
        seeked_districts: ["Sceaux",
        "Boulogne-Billancourt",
        "Antony",
        "Bourg-la-Reine",
        "Paris"],
        category_type: 3,
        single_parent: false,
        widen_request: true,
        housing_type_list: ["LP"],
        seeked_district_postal_codes: ["92330",
        "92100",
        "92160",
        "92340",
        "75000"],
        seeked_departments: ["92",
        "92",
        "92",
        "92"],
        seeked_localisation_list: [{"nom"=>"Sceaux",
        "code"=>"92071",
        "codeDepartement"=>"92",
        "codeRegion"=>"11",
        "codesPostaux"=>["92330"],
        "population"=>19479,
        "_score"=>0.4094480046151159},
        {"nom"=>"Boulogne-Billancourt",
        "code"=>"92012",
        "codeDepartement"=>"92",
        "codeRegion"=>"11",
        "codesPostaux"=>["92100",
        "75016"],
        "population"=>119645,
        "_score"=>0.6337893925744263},
        {"nom"=>"Antony",
        "code"=>"92002",
        "codeDepartement"=>"92",
        "codeRegion"=>"11",
        "codesPostaux"=>["92160"],
        "population"=>62210,
        "_score"=>0.21006442878421805},
        {"nom"=>"Bourg-la-Reine",
        "code"=>"92014",
        "codeDepartement"=>"92",
        "codeRegion"=>"11",
        "codesPostaux"=>["92340"],
        "population"=>20531,
        "_score"=>0.1047225184514744},
        {"nom"=>"Paris",
        "code"=>"75",
        "codeRegion"=>"11",
        "_score"=>1}],
        requester_firstname: "Julie",
        requester_lastname: "RAHMA",
        requester_fullname: "Julie RAHMA",
        requester_email: "julie.rahma@yopmail.com",
        requester_birth_date: '1995-08-08 23:00:00 UTC',
        monthly_revenues: 2825.0,
        typology_range: ["T2",
        "T3",
        "T4"],
        family_size: 3,
        unborn_children: [{}],
        actual_housing_details: {"postal_code"=>"78100",
        "number"=>"147",
        "way"=>"rue de sevères",
        "district"=>"SAINT GERMAIN EN LAYE",
        "address_insee_code"=>"78100"},
        postal_details: {},
        owned_housing_details: {
          "type"=>nil,
          "postal_code"=>nil,
          "district"=>nil
        },
        employee_id: "MLS",
        employee_start_date: '2018-01-01 00:00:00 UTC',
        entity_access_level: "total",
        tab_state: "0",
        motivation1: "02",
        motivation2: "06",
        seeked_housing_categories: [""],
        min_surface: 25,
        dependents_monthly_incomes: {},
        dependents_monthly_charges: {},
        out_of_range_types: ["T2",
        "T3",
        "T4"],
        applications_nb: 0,
        name_has_changed: false,
        birth_date_has_changed: false,
        monthly_revenues_will_change: false,
        category_may_change: false,
        should_update_eligibility: false,
        tax_revenues_will_change: false,
        superfields: {"last_coapplicants"=>[{"rel_type"=>"P",
        "id"=>"5e20432bb93ee317fbf72887"}]},
        applicant_ids: [ambroiseRAHMA.id],
        housing_offer_ids: [],
        entity_status: "A VALIDER",
        request_status: "COMPLETE",
        nud_creations: {},
        main_applicant_id: julieRAHMA.id,
      )
    end
      
    describe 'Eligibility query are valid' do
      it 'family2' do
        offers_family2 = Hermes::HousingOffer.where(applicant2a.reload.eligibility_query.json_to_h)
        offers_ids = offers_family2.pluck(:id)
        expect(offers_family2.length).to eq(4)
        [housing_offer_L1, housing_offer_L2, housing_offer_L10, housing_offer_L11].each do |offer|
          expect(offers_ids).to include(offer.id)
        end
      end

      it 'RaphaelGOUBILL' do
        offers = Hermes::HousingOffer.where(raphaelGOUBILL.reload.eligibility_query.json_to_h)
        offers_ids = offers.pluck(:id)

        expect(offers.length).to eq(4)
        [housing_offer_L1, housing_offer_L2, housing_offer_L10, housing_offer_L11].each do |offer|
          expect(offers_ids).to include(offer.id)
        end
      end

      it 'laureGILLET' do
        offers = Hermes::HousingOffer.where(laureGILLET.reload.eligibility_query.json_to_h)
        offers_ids = offers.pluck(:id)
        expect(offers.length).to eq(2)
        [housing_offer_L4, housing_offer_L11].each do |offer|
          expect(offers_ids).to include(offer.id)
        end
      end

      it 'simonDURANDET' do
        offers = Hermes::HousingOffer.where(simonDURANDET.reload.eligibility_query.json_to_h)
        offers_ids = offers.pluck(:id)

        expect(offers.length).to eq(3)
        [housing_offer_L1, housing_offer_L2, housing_offer_L10].each do |offer|
          expect(offers_ids).to include(offer.id)
        end
      end

      it 'manonGATTIN' do
        offers = Hermes::HousingOffer.where(manonGATTIN.reload.eligibility_query.json_to_h)
        offers_ids = offers.pluck(:id)

        expect(offers.length).to eq(0)
      end

      it 'julieRAHMA' do
        offers = Hermes::HousingOffer.where(julieRAHMA.reload.eligibility_query.json_to_h)
        offers_ids = offers.pluck(:id)


        expect(offers.length).to eq(4)
        [housing_offer_L1, housing_offer_L2, housing_offer_L10, housing_offer_L11].each do |offer|
          expect(offers_ids).to include(offer.id)
        end
      end
    end

    describe 'Geocoding' do
      let!(:applicant) { create(:applicant, address: "vieux chemin d'arles", postal_code: '13210') }
      it 'should call geocoder if address change' do
        applicant.address = 'routes des palus'
        applicant.save!
        applicant.reload

        job = ActiveJob::Base.queue_adapter.enqueued_jobs.last
        expect(job[:job]).to eq(GeocodeJob)
        expect(job[:queue]).to eq('geocoder')
        expect(job[:args]).to eq(
          [
            applicant.class.name,
            applicant.id.to_s,
            true
          ]
        )
      end
    end

    describe 'Class methods' do
      let(:applicant) { create(:applicant) }
      let(:housing_request) { create(:housing_request, main_applicant_id: applicant.id) }
      let(:nud) { create(:nud, unique_number: "111101657788975B02", applicant: applicant, housing_request: housing_request) }
      it 'should find applicant with existing nud' do
        expect(Hermes::Applicant.find_or_create_by_nud(nud.unique_number)).to eq(applicant)
      end

      let(:unique_number) { '111101779425478E02' }
      it 'should create applicant with no existing nud and valid birthdate' do
        created_applicant = Hermes::Applicant.find_or_create_by_nud(unique_number, '1991-05-10')
        created_nud = Hermes::Nud.find_by(unique_number: unique_number)
        expect(created_applicant).to be_an_instance_of(Hermes::Applicant)
        expect(created_applicant.user).not_to be_nil
        expect(created_applicant.housing_requests.count).to eq(1)
        expect(created_nud.applicant).to eq(created_applicant)
      end

      it 'should not create applicant with no existing nud and invalid birthdate' do
        created_applicant = Hermes::Applicant.find_or_create_by_nud(unique_number, '2001-05-10')
        expect(created_applicant.errors.messages[:birth_date]).to eq ["La date de naissance saisie, ne corresponda pas avec c'elle du demandeur"]
      end

      it 'should not create applicant with invalid nud' do
        created_applicant = Hermes::Applicant.find_or_create_by_nud('000000000000000000', '2001-05-10')
        expect(created_applicant.errors.messages[:nud]).to eq ["Le nud saisi est invalide"]
      end
    end
  end

  describe 'test change name' do
    let!(:app) { create(:applicant, first_name: 'John', last_name: 'Doe', email: 'johndoe@mail.com')}
    let!(:hr) { create(:housing_request, main_applicant: app, is_active: true)}

    it 'setup' do 
      hr.reload
      expect(hr.requester_firstname).to eq(app.first_name)
      expect(hr.requester_lastname).to eq(app.last_name)
      expect(hr.requester_email).to eq(app.email)
    end 

    it 'update email' do 
      app.update(email: 'lol@mail.com')
      hr.reload
      expect(hr.requester_email).to eq('lol@mail.com')
    end 
  end 

  describe 'old_entity_id' do 
    let!(:entity)  { create(:entity, siret: '00000000000001')}
    let!(:entity2)  { create(:entity, siret: '00000000000002')}
    let!(:applicant) { create(:applicant, main_employer_siret: entity.siret)}
    let!(:hr) { create(:housing_request, is_active: true, main_applicant: applicant)}

    it 'change when entity_change' do 
      applicant.update(main_employer_siret: entity2.siret)
      expect(applicant.reload.entity_id).to eq(entity2.id)
      expect(applicant.reload.old_entity_id).to eq(entity.id)
    end 

    it 'change when wrong siret' do 
      applicant.update(main_employer_siret: nil)
      applicant.update(main_employer_siret: "00000000000011")
      expect(applicant.reload.entity_id).to eq(nil)
      expect(applicant.reload.old_entity_id).to eq(nil)
    end     

  end 
end
